@@ -1,32 +1,26 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
-* Copyright 2013 - 2021, nymea GmbH
-* Contact: contact@nymea.io
+* Copyright (C) 2013 - 2024, nymea GmbH
+* Copyright (C) 2024 - 2025, chargebyte austria GmbH
 *
-* This fileDescriptor is part of nymea.
-* This project including source code and documentation is protected by
-* copyright law, and remains the property of nymea GmbH. All rights, including
-* reproduction, publication, editing and translation, are reserved. The use of
-* this project is subject to the terms of a license agreement to be concluded
-* with nymea GmbH in accordance with the terms of use of nymea GmbH, available
-* under https://nymea.io/license
+* This file is part of libnymea-sunspec.
 *
-* GNU Lesser General Public License Usage
-* Alternatively, this project may be redistributed and/or modified under the
-* terms of the GNU Lesser General Public License as published by the Free
-* Software Foundation; version 3. This project is distributed in the hope that
-* it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-* Lesser General Public License for more details.
+* libnymea-sunspec is free software: you can redistribute it and/or
+* modify it under the terms of the GNU Lesser General Public License
+* as published by the Free Software Foundation, either version 3
+* of the License, or (at your option) any later version.
+*
+* libnymea-sunspec is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+* GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
-* along with this project. If not, see <https://www.gnu.org/licenses/>.
+* along with libnymea-sunspec. If not, see <https://www.gnu.org/licenses/>.
 *
-* For any further details and any questions please contact us under
-* contact@nymea.io or see our FAQ/Licensing Information on
-* https://nymea.io/license/faq
-*
-* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #include "sunspechvrtxmodel.h"
 #include "sunspecconnection.h"
@@ -34,6 +28,7 @@
 SunSpecHvrtxModelRepeatingBlock::SunSpecHvrtxModelRepeatingBlock(quint16 blockIndex, quint16 blockSize, quint16 modbusStartRegister, SunSpecHvrtxModel *parent) :
     SunSpecModelRepeatingBlock(blockIndex, blockSize, modbusStartRegister, parent)
 {
+    m_parentModel = parent;
     m_byteOrder = parent->byteOrder();
     initDataPoints();
 }
@@ -1326,10 +1321,8 @@ void SunSpecHvrtxModelRepeatingBlock::initDataPoints()
 
 }
 
-void SunSpecHvrtxModelRepeatingBlock::processBlockData(const QVector<quint16> blockData)
+void SunSpecHvrtxModelRepeatingBlock::processBlockData()
 {
-    m_blockData = blockData;
-
     // Update properties according to the data point type
     if (m_dataPoints.value("ActPt").isValid())
         m_actPt = m_dataPoints.value("ActPt").toUInt16();
@@ -1471,6 +1464,8 @@ SunSpecHvrtxModel::SunSpecHvrtxModel(SunSpecConnection *connection, quint16 modb
     m_modelBlockType = SunSpecModel::ModelBlockTypeFixedAndRepeating;
 
     initDataPoints();
+
+    connect(this, &SunSpecModel::initFinished, this, &SunSpecHvrtxModel::setupRepeatingBlocks);
 }
 
 SunSpecHvrtxModel::~SunSpecHvrtxModel()
@@ -1797,6 +1792,36 @@ void SunSpecHvrtxModel::processBlockData()
     qCDebug(dcSunSpecModelData()) << this;
 }
 
+void SunSpecHvrtxModel::setupRepeatingBlocks()
+{
+    if (!m_repeatingBlocks.isEmpty()) {
+        foreach (SunSpecModelRepeatingBlock *block, m_repeatingBlocks) {
+            block->deleteLater();
+        }
+        m_repeatingBlocks.clear();
+    }
+
+    const auto headerLength = 2;
+    const auto repeatingBlocksDataSize = m_blockData.size() - headerLength - m_fixedBlockLength;
+    if (repeatingBlocksDataSize % m_repeatingBlockLength != 0) {
+        qCWarning(dcSunSpecModelData()) << "Unexpected repeating block data size:"
+                                        << repeatingBlocksDataSize
+                                        << "(repeating block size:"
+                                        << m_repeatingBlockLength
+                                        << ", extra bytes:"
+                                        << repeatingBlocksDataSize % m_repeatingBlockLength
+                                        << "). Repeating blocks will not be handled!";
+        return;
+    }
+    const auto numberOfBlocks = repeatingBlocksDataSize / m_repeatingBlockLength;
+    const auto repeatingBlocksOffset = m_fixedBlockLength + headerLength;
+    for (int i = 0; i < numberOfBlocks; ++i) {
+        const auto blockStartRegister = static_cast<quint16>(modbusStartRegister() + repeatingBlocksOffset + m_repeatingBlockLength * i);
+        const auto block = new SunSpecHvrtxModelRepeatingBlock(i, m_repeatingBlockLength, blockStartRegister, this);
+        m_repeatingBlocks.append(block);
+    }
+}
+
 QDebug operator<<(QDebug debug, SunSpecHvrtxModel *model)
 {
     debug.nospace().noquote() << "SunSpecHvrtxModel(Model: " << model->modelId() << ", Register: " << model->modbusStartRegister() << ", Length: " << model->modelLength() << ")\n";
